/*
 * @(#)java_md.c	1.9 98/06/29
 *
 * Copyright 1997, 1998 by Sun Microsystems, Inc.,
 * 901 San Antonio Road, Palo Alto, California, 94303, U.S.A.
 * All rights reserved.
 *
 * This software is the confidential and proprietary information
 * of Sun Microsystems, Inc. ("Confidential Information").  You
 * shall not disclose such Confidential Information and shall use
 * it only in accordance with the terms of the license agreement
 * you entered into with Sun.
 */

#define _CRT_SECURE_NO_WARNINGS

#include <windows.h>
#include <stdlib.h>
#include <stdio.h>
#include <sys/types.h>
#include <sys/stat.h>

#include <jni.h>
#include "java.h"

#include "service.h"

#ifdef DEBUG
#define JVM_DLL "jvm_g.dll"
#define JAVA_DLL "java_g.dll"
#else
#define JVM_DLL "jvm.dll"
#define JAVA_DLL "java.dll"
#endif
/*
* Helpers to look in the registry for a public JRE.
*/
/*
#define ALT2DOTRELEASE  "1.2" 
#define ALT3DOTRELEASE  "1.3" 
#define ALT4DOTRELEASE  "1.4" 
#define ALT5DOTRELEASE  "1.5" 
#define ALT6DOTRELEASE  "1.6" 
#define ALT7DOTRELEASE  "1.7" 
#define ALT8DOTRELEASE  "1.8" 
#define ALT8DOTRELEASEZULU  "8." 
#define ALT9DOTRELEASE  "9." 
*/
#define JRE_KEY_ZULU_17 "Software\\Azul Systems\\Zulu\\zulu-17"
#define JRE_KEY_ZULU_16 "Software\\Azul Systems\\Zulu\\zulu-16"
#define JRE_KEY_ZULU_15 "Software\\Azul Systems\\Zulu\\zulu-15"
#define JRE_KEY_ZULU_14 "Software\\Azul Systems\\Zulu\\zulu-14"
#define JRE_KEY_ZULU_13 "Software\\Azul Systems\\Zulu\\zulu-13"
#define JRE_KEY_ZULU_12 "Software\\Azul Systems\\Zulu\\zulu-12"
#define JRE_KEY_ZULU_11 "Software\\Azul Systems\\Zulu\\zulu-11"
#define JRE_KEY_ZULU_10 "Software\\Azul Systems\\Zulu\\zulu-10"
#define JRE_KEY_ZULU_9 "Software\\Azul Systems\\Zulu\\zulu-9"
#define JRE_KEY_ZULU_8 "Software\\Azul Systems\\Zulu\\zulu-8"
#define JRE_KEY_9	    "Software\\JavaSoft\\JRE"
#define JDK_KEY_9	    "Software\\JavaSoft\\JDK"
#define JRE_KEY	    "Software\\JavaSoft\\Java Runtime Environment"
#define JDK_KEY	    "Software\\JavaSoft\\Java Development Kit"


#define ORACLE  1 
#define ZULU  2 

/*
 * Prototypes.
 */
static jboolean GetPublicJREHomeCurrent(char *buf, jint bufsize, char* reg_key, jint type, char* allowedversion);
static jboolean GetPublicJREHomeAll(char *buf, jint bufsize, char* reg_key, jint type, char* allowedversion);
jboolean CheckJavaVersion(char* version, char* allowed);

/*
 * Load JVM of "jvmtype", and intialize the invocation functions.  Notice that
 * if jvmtype is NULL, we try to load hotspot, client or classic VM as the default.  Maybe we
 * need an environment variable that dictates the choice of default VM.
 * As of version 0.99l jvmtype can be a comma seperated list. Effectively making
 * the above hotspot default obsolete as they can now be specified explicitely in order of preference
 */
jboolean
LoadJavaVM(char *jvmtype, InvocationFunctions *ifn)
{
	TCHAR szErr[256];
    char home[MAXPATHLEN], javadll[MAXPATHLEN], jvmdll[MAXPATHLEN];
    HINSTANCE handle;
    struct stat s;
	size_t pathSize = 32767;
	size_t i;
	LPTSTR path;
	LPTSTR* jvmtypes = NULL;

	//Empty string in home
	home[0] = 0;

	/* Is JRE defined in ini file? */
	if (jreSearchPath == TRUE)
	{
		if (bDebug)
			logToFile("Searching for JVM in path from ini file");
		if (strlen(global_jrePath) > 0)
		{
			(void)strcpy(home, global_jrePath);
			sprintf(javadll, "%s\\bin\\" JAVA_DLL, home);
			if (bDebug)
				logToFile(javadll);
			if (stat(javadll, &s) == 0)
			{
				if (bDebug)
					logToFile("Found JVM in path from ini file");
				goto jrefound;
			}
		}

		if (strlen(global_jrePath) > 0)
		{
			(void)strcpy(home, global_jrePath);
			sprintf(javadll, "%s\\jre\\bin\\" JAVA_DLL, home);
			if (bDebug)
				logToFile(javadll);
			if (stat(javadll, &s) == 0)
			{
				strcat(home, "\\jre");
				if (bDebug)
					logToFile("Found JVM path from ini file");
				goto jrefound;
			}
		}
		//end 01.03.2001 Erwin Vervondel
	}

	if (jreSearchLocal == TRUE)
	{
		/* Is JRE co-located with the application? */
		if (bDebug)
			logToFile("Searching for JVM in ./bin");
		if (GetApplicationHome(home, sizeof(home))) {
			sprintf(javadll, "%s\\bin\\" JAVA_DLL, home);
			if (bDebug)
				logToFile(javadll);
			if (stat(javadll, &s) == 0)
			{
				if (bDebug)
					logToFile("Found JVM in ./bin");
				goto jrefound;
			}
		}
		/* Does this app ship a private JRE in <apphome>\jre directory? */
		if (bDebug)
			logToFile("Searching for JVM in ./jre/bin");
		sprintf(javadll, "%s\\jre\\bin\\" JAVA_DLL, home);
		if (bDebug)
			logToFile(javadll);
		if (stat(javadll, &s) == 0)
		{
			strcat(home, "\\jre");
			if (bDebug)
				logToFile("Found JVM in ./jre/bin");
			goto jrefound;
		}
	}

	//registryfindadoptopenjdk and registryfindj9 are equivalent to registryfindoracle
	registryfindoracle = registryfindoracle || registryfindadoptopenjdk || registryfindj9;

	//Now loop through all allowed versions looking in the registry
	//TODO: Need to loop through a range for Zulu ... 
	for (int i = 0; jreVersions[i] != NULL; i++) 
	{
		/* Look for a public ZULU 17 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 17 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_17, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public ZULU 16 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 16 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_16, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public ZULU 15 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 15 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_15, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public ZULU 14 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 14 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_14, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public ZULU 13 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 13 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_13, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public ZULU 12 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 12 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_12, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public ZULU 11 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 11 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_11, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public ZULU 10 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 10 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_10, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}

		/* Look for a public ZULU 9 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 9 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_9, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}

		/* Look for a public Oracle or OpenJDK JRE 9 or later on this machine. */
		if (jreSearchRegistry == TRUE && registryfindjre == TRUE && registryfindoracle == TRUE)
		{
			if (bDebug)
			{
				logToFile("Searching Oracle or OpenJDK 9 JRE or later in registry");
				//printf("%s=%d\n", "registryfindjre", registryfindjre);
			}

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_9, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}
			if (GetPublicJREHomeAll(home, sizeof(home), JRE_KEY_9, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}

		}

		/* Look for a public Oracle or OpenJDK JDK 9 or later on this machine. */
		if (jreSearchRegistry == TRUE && registryfindjdk == TRUE && registryfindoracle == TRUE)
		{
			if (bDebug)
				logToFile("Searching Oracle or OpenJDK 9 JDK or later in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JDK_KEY_9, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}
			if (GetPublicJREHomeAll(home, sizeof(home), JDK_KEY_9, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}

		}
		/* Look for a public ZULU 8 JRE on this machine. */
		if (jreSearchRegistry == TRUE && registryfindazul == TRUE)
		{
			if (bDebug)
				logToFile("Searching ZULU 8 in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY_ZULU_8, ZULU, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public Oracle JRE 8 or older on this machine. */
		if (jreSearchRegistry == TRUE && registryfindjre == TRUE && registryfindoracle == TRUE)
		{
			if (bDebug)
				logToFile("Searching Oracle 8 JRE or older in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JRE_KEY, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}
			if (GetPublicJREHomeAll(home, sizeof(home), JRE_KEY, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}
		}
		/* Look for a public Oracle JDK 8 or older on this machine. */
		if (jreSearchRegistry == TRUE && registryfindjdk == TRUE && registryfindoracle == TRUE)
		{
			if (bDebug)
				logToFile("Searching Oracle 8 JDK or older in registry");

			if (GetPublicJREHomeCurrent(home, sizeof(home), JDK_KEY, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}
			if (GetPublicJREHomeAll(home, sizeof(home), JDK_KEY, ORACLE, jreVersions[i]))
			{
				goto jrefound;
			}
		}
	}
	 
	//If we did not find a JVM
	
	logToFile("ERROR: Did no find any JVM in any search locations");
	return JNI_FALSE;
	

    /* Now we know where JRE is -- the value in the "home" variable. */
jrefound:    
	if (bDebug)
		logToFile("Found JVM in registry");
	//Allow jvmtype to be specified from ini file
	//But first we check of the jvmtype was specified on command line
	if ( jvmtype == NULL )
	{
		jvmtype = global_jvmtype; 
	}
	
	if (bDebug)
	{
		logToFile("Allowed JVM types");
		logToFile(jvmtype);
	}
	jvmtypes = stringToArray( jvmtype, ',' );
	
	//Now we run through the list of jvmtypes until we find one that is installed	
	if ( jvmtypes != NULL )
	{
		
		jvmtype = NULL;
		for( i=0; jvmtype==NULL && jvmtypes[i]!=NULL; i++)
		{
			sprintf(jvmdll, "%s\\bin\\%s\\" JVM_DLL, home, jvmtypes[i]);
			if (stat(jvmdll, &s) >= 0) 
			{
				jvmtype = jvmtypes[i];
				if ( bDebug )
				{
					logToFile("Found allowed JVM type");
					logToFile( jvmdll );
				}
			}
			else {
				sprintf(jvmdll, "%s\\jre\\bin\\%s\\" JVM_DLL, home, jvmtypes[i]);
				if (stat(jvmdll, &s) >= 0)
				{
					jvmtype = jvmtypes[i];
					if (bDebug)
					{
						logToFile("Found allowed JVM type");
						logToFile(jvmdll);
					}
				}
			}
		}
		if ( jvmtype == NULL )
		{
			fprintf(stderr, "Error trying to locate any of the JVM types listed below\n");
			for( i=0; jvmtypes[i]!=NULL; i++)
				fprintf(stderr, "Failed to find jvmtype = %s\n", jvmtypes[i] );
			jvmtype = "notinstalledJVMType";
		}
	}
	
		
	/* Now we know what the jvmtype should be */
    /*
	sprintf(jvmdll, "%s\\bin\\%s\\" JVM_DLL, home, jvmtype);
    if (bDebug) {	
		logToFile( "JVM DLL:" );
		logToFile( jvmdll );
    }
	*/
	
	//2009.10.24 Fix for msvcrt71.dll dependency. Add JVM home to PATH
	//Should be limited to 32bit

	path = malloc(pathSize);
#pragma warning(disable:4267)
	GetEnvironmentVariable("Path", path, pathSize);
#pragma warning(default:4267)
	//printf("Path: %s\n", path);
	strcat_s(path, pathSize, ";");
	strcat_s(path, pathSize, home);
	strcat_s(path, pathSize, "\\bin");
	SetEnvironmentVariable("Path", path);

	if (bDebug)
	{
		logToFile("Path Variable:");
		logToFile(path);
	}


    /* Load the Java VM DLL */
    if ((handle = LoadLibrary(jvmdll)) == 0) {
		
		logToFile(GetLastErrorText(szErr, 256));
		logToFile("Error loading JVM");
		logToFile(jvmdll);
		
		fprintf(stderr, "Error loading: %s - Please check the jvmtype parameter in the configuration file.\n", jvmdll);
		if (stat(jvmdll, &s) >= 0)
		{
			fprintf(stderr, "JVM is present but did not load. Please make sure you are using jsl.exe for a 32bit JVM and jsl64.exe for a 64bit JVM.\n");
			#ifdef _WIN64
				logToFile("This is a 64bit jsl64.exe");
				fprintf(stderr, "This is a 64bit jsl64.exe.\n");
			#else
				logToFile("This is a 32bit jsl.exe");
				fprintf(stderr, "This is a 32bit jsl.exe.\n");
			#endif
		}
	return JNI_FALSE;
    }

    /* Now get the function addresses */
    ifn->CreateJavaVM = (void *)GetProcAddress(handle, "JNI_CreateJavaVM");
    ifn->GetDefaultJavaVMInitArgs = (void *)GetProcAddress(handle, "JNI_GetDefaultJavaVMInitArgs");
    if (ifn->CreateJavaVM == 0 || ifn->GetDefaultJavaVMInitArgs == 0) {
		fprintf(stderr, "Can't find JNI interfaces in: %s\n", jvmdll);
		return JNI_FALSE;
    }

    return JNI_TRUE;
}

/*
 * Get the path to the file that has the usage message for -X options.
 */
void
GetXUsagePath(char *buf, jint bufsize)
{
    GetModuleFileName(GetModuleHandle(JVM_DLL), buf, bufsize);
    *(strrchr(buf, '\\')) = '\0';
    strcat(buf, "\\Xusage.txt");
}

/*
 * If app is "c:\foo\bin\javac", then put "c:\foo" into buf.
 */
jboolean
GetApplicationHome(char *buf, jint bufsize)
{
    char *cp;
    GetModuleFileName(0, buf, bufsize);
    *strrchr(buf, '\\') = '\0'; /* remove .exe file name */
    if ((cp = strrchr(buf, '\\')) == 0) {
	/* This happens if the application is in a drive root, and
	 * there is no bin directory. */
	buf[0] = '\0';
	return JNI_FALSE;
    }
    *cp = '\0';  /* remove the bin\ part */
    return JNI_TRUE;
}

#ifdef JAVAW
__declspec(dllimport) char **__initenv;

int WINAPI
WinMain(HINSTANCE inst, HINSTANCE previnst, LPSTR cmdline, int cmdshow)
{
    __initenv = _environ;
    return main(__argc, __argv);
}
#endif



static jboolean
GetStringFromRegistry(HKEY key, const char *name, char *buf, jint bufsize)
{
    DWORD type, size;

    if (RegQueryValueEx(key, name, 0, &type, 0, &size) == 0 && type == REG_SZ && (size < (unsigned int)bufsize)) {
		if (RegQueryValueEx(key, name, 0, 0, buf, &size) == 0) {
			return JNI_TRUE;
		}
    }
    return JNI_FALSE;
}

static jboolean GetPublicJREHomeAll(char *buf, jint bufsize, char* reg_key, jint type, char* allowed)
{
	HKEY key = NULL, subkey = NULL;
	DWORD idx = 0;
	char version[MAXPATHLEN];
	DWORD versionl = MAXPATHLEN;
	jboolean found = FALSE;

	/* Open the JRE registry root*/
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, reg_key, 0, KEY_READ, &key) != 0) {
		if (bDebug)
			fprintf(stderr, "Error opening registry key '%s'\n", reg_key);
		return JNI_FALSE;
	}

	while (!found && ERROR_SUCCESS == RegEnumKeyEx(key, idx, version, &versionl, NULL, NULL, NULL, NULL))
	{
		//Match against allowed versions
		found = CheckJavaVersion(version, allowed );
		
		if (type == ORACLE) {
			/* Find directory where the current version is installed. */
			if (RegOpenKeyEx(key, version, 0, KEY_READ, &subkey) != 0) {
				fprintf(stderr, "Error opening registry key '%s\\%s'\n", reg_key, version);
				RegCloseKey(key);
				return JNI_FALSE;
			}
			
			if (!GetStringFromRegistry(subkey, "JavaHome", buf, bufsize)) {
				fprintf(stderr, "Error reading value of registry key:\n\t %s\\%s\\JavaHome\n", reg_key, version);
				RegCloseKey(key);
				RegCloseKey(subkey);
				return JNI_FALSE;
			}
			RegCloseKey(subkey);
		}
		versionl = MAXPATHLEN;
		idx++;
	}

	RegCloseKey(key);

	return found;
}

static jboolean GetPublicJREHomeCurrent(char *buf, jint bufsize, char* reg_key, jint type, char* allowed)
{
    HKEY key = NULL, subkey = NULL;
    char version[MAXPATHLEN];
	jboolean found = FALSE;

    /* Find the current version of the JRE */
	if (RegOpenKeyEx(HKEY_LOCAL_MACHINE, reg_key, 0, KEY_READ, &key) != 0) {
		if (bDebug)
			fprintf(stderr, "Error opening registry key '%s'\n", reg_key);
		return JNI_FALSE;
    }

    if (!GetStringFromRegistry(key, "CurrentVersion", version, sizeof(version))) {
		if (bDebug)
			fprintf(stderr, "Error reading value of registry key:\n\t %s\\CurrentVersion\n", reg_key);
		RegCloseKey(key);
		return JNI_FALSE;
    }

	/* This check has become redundant - In the more common case of directly pointing to the JRE on disk its not used anyway
	if  ((strncmp(version, ALT2DOTRELEASE,2) != 0) &&
		(strncmp(version, ALT3DOTRELEASE, 2) != 0) &&
		(strncmp(version, ALT4DOTRELEASE, 2) != 0) &&
		(strncmp(version, ALT5DOTRELEASE, 2) != 0) &&
		(strncmp(version, ALT6DOTRELEASE, 2) != 0) &&
		(strncmp(version, ALT7DOTRELEASE, 2) != 0) &&
		(strncmp(version, ALT8DOTRELEASE, 2) != 0) &&
		(strncmp(version, ALT8DOTRELEASEZULU, 2) != 0) &&
		(strncmp(version, ALT9DOTRELEASE, 2) != 0)
	   )
	{
	//Invalid JVM version is no longer a error, just a warning.
	fprintf(stderr, "WARNING: Registry key %s\\CurrentVersion'\n has "
		"value '%s', but only '" ALT2DOTRELEASE " ," ALT3DOTRELEASE " ," ALT4DOTRELEASE " ," ALT5DOTRELEASE " ," ALT6DOTRELEASE " ," ALT7DOTRELEASE " ," ALT8DOTRELEASE ", " ALT8DOTRELEASEZULU " and " ALT9DOTRELEASE "' have been tested.\n", 
		reg_key, version);
    }
	*/

	if (type == ORACLE) {
		/* Find directory where the current version is installed. */
		if (RegOpenKeyEx(key, version, 0, KEY_READ, &subkey) != 0) {
			fprintf(stderr, "Error opening registry key '%s\\%s'\n", reg_key, version);
			RegCloseKey(key);
			return JNI_FALSE;
		}
		if (!GetStringFromRegistry(subkey, "JavaHome", buf, bufsize)) {
			fprintf(stderr, "Error reading value of registry key:\n\t %s\\%s\\JavaHome\n", reg_key, version);
			RegCloseKey(key);
			RegCloseKey(subkey);
			return JNI_FALSE;
		}
		RegCloseKey(subkey);
	}
	if (type == ZULU) {
		if (!GetStringFromRegistry(key, "InstallationPath", buf, bufsize)) {
			fprintf(stderr, "Failed reading value of registry key:\n\t %s\\%s\\InstallationPath\n", reg_key, version);
			RegCloseKey(key);
			return JNI_FALSE;
		}
	}

	/* Debug code of limited use
    if (debug) {
		char micro[MAXPATHLEN];
		if (!GetStringFromRegistry(subkey, "MicroVersion", micro,sizeof(micro))) {
			printf("Warning: Can't read MicroVersion\n");
			micro[0] = '\0';
		}
		printf("Version major.minor.micro = %s.%s\n", version, micro);
    }
	*/

    RegCloseKey(key);
   

	//Match against allowed versions
	found = CheckJavaVersion(version, allowed);

	return found;
}


jboolean CheckJavaVersion(char* version, char* allowed)
{
	jboolean bjreversionallowed = FALSE;
	size_t max;

	if (bDebug) {
		logToFile("Checking Java Version:");
		logToFile(version);
		logToFile(allowed);
	}
	max = strlen(allowed);
	if (strncmp(allowed, version, max) == 0) bjreversionallowed = TRUE;
	if (strncmp(allowed, "ANY", strlen(allowed)) == 0) bjreversionallowed = TRUE;

	if (bDebug == TRUE && bjreversionallowed == TRUE) {
		logToFile("Matching:");
		logToFile(allowed);
	}
	return bjreversionallowed;
}

jboolean CheckJavaVersionAll(char* javaversion)
{
	jboolean bjreversionallowed = FALSE;
	size_t max;
	//Check for allowed Java versions
	if (bDebug) {
		logToFile("Checking Java Version:");
		logToFile(javaversion);
	}

	for (int i = 0; bjreversionallowed == FALSE && jreVersions[i] != NULL; i++) {
		max = strlen(jreVersions[i]);
		if (strlen(javaversion) < max) max = strlen(javaversion);
		if (strncmp(jreVersions[i], javaversion, max) == 0) bjreversionallowed = TRUE;
		if (strncmp(jreVersions[i], "ANY", strlen(jreVersions[i])) == 0) bjreversionallowed = TRUE;
		if (bDebug == TRUE && bjreversionallowed == TRUE) {
			logToFile("Matching:");
			logToFile(jreVersions[i]);
		}
	}
	if (!bjreversionallowed)
	{
		logToFile("No matches.");
	}

	return bjreversionallowed;
}


/*
 * Support for doing cheap, accurate interval timing.
 */
static jboolean counterAvailable = JNI_FALSE;
static jboolean counterInitialized = JNI_FALSE;
static LARGE_INTEGER counterFrequency;

jlong CounterGet()
{
    LARGE_INTEGER count;

    if (!counterInitialized) {
	counterAvailable = QueryPerformanceFrequency(&counterFrequency);
	counterInitialized = JNI_TRUE;
    }
    if (!counterAvailable) {
	return 0;
    }
    QueryPerformanceCounter(&count);
    return (jlong)(count.QuadPart);
}

jlong Counter2Micros(jlong counts)
{
    if (!counterAvailable || !counterInitialized) {
	return 0;
    }
    return (counts * 1000 * 1000)/counterFrequency.QuadPart;
}
